@@ -55,6 +55,7 @@
 #include "speller_impl.hpp"
 #include "vararray.hpp"
 #include "lsort.hpp"
+#include "hash-t.hpp"
 
 using namespace std;
 
@@ -68,16 +69,25 @@ static char EMPTY[1] = {0};
 //*Entry struct definations
 //
 
+struct Conds
+{
+  char * str;
+  unsigned num;
+  char conds[SETSIZE];
+  char get(byte i) const {return conds[i];}
+};
+
 struct AffEntry
 {
   char *         appnd;
   char *         strip;
   unsigned int   appndl;
   unsigned int   stripl;
-  unsigned int   numconds;
   int            xpflg;
-  char         achar;
-  char         conds[SETSIZE];
+  char           achar;
+  const Conds *  conds;
+  //unsigned int numconds;
+  //char         conds[SETSIZE];
 };
 
 // A Prefix Entry
@@ -97,6 +107,7 @@ struct PfxEntry : public AffEntry
   inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
   inline const char *  key() const  { return appnd;  }
+  bool applicable(SimpleString) const;
   SimpleString add(SimpleString, ObjStack & buf) const;
 };
 
@@ -120,6 +131,7 @@ struct SfxEntry : public AffEntry
   inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
   inline const char *  key() const  { return rappnd; } 
+  bool applicable(SimpleString) const;
   SimpleString add(SimpleString, ObjStack & buf, int limit, SimpleString) const;
 };
 
@@ -155,6 +167,21 @@ struct AffixLess
   bool operator() (T * x, T * y) const {return strcmp(x->key(),y->key()) < 0;}
 };
 
+struct CondsLookupParms {
+  typedef const Conds * Value;
+  typedef const char * Key;
+  static const bool is_multi = false;
+  hash<const char *> hfun;
+  size_t hash(const char * s) {return hfun(s);}
+  bool equal(const char * x, const char * y) {return strcmp(x,y) == 0;}
+  const char * key(const Conds * c) {return c->str;}
+};
+
+typedef HashTable<CondsLookupParms> CondsLookup;
+
+static void encodeit(CondsLookup &, ObjStack &, 
+                     AffEntry * ptr, char * cs);
+
 //////////////////////////////////////////////////////////////////////
 //
 // Affix Manager
@@ -175,34 +202,10 @@ PosibErr<void> AffixMgr::setup(ParmString affpath, Conv & iconv)
   return parse_file(affpath, iconv);
 }
 
-AffixMgr::~AffixMgr()
-{
-  // pass through linked prefix entries and clean up
-  for (int i=0; i < SETSIZE ;i++) {
-    pFlag[i] = NULL;
-    PfxEntry * ptr = pStart[i];
-    PfxEntry * nptr = NULL;
-    while (ptr) {
-      nptr = ptr->next;
-      delete ptr;
-      ptr = nptr;
-      nptr = NULL;
-    }
-  }
+AffixMgr::AffixMgr(const Language * l) 
+  : lang(l), data_buf(1024*16) {}
 
-  // pass through linked suffix entries and clean up
-  for (int j=0; j < SETSIZE ; j++) {
-    sFlag[j] = NULL;
-    SfxEntry * ptr = sStart[j];
-    SfxEntry * nptr = NULL;
-    while (ptr) {
-      nptr = ptr->next;
-      delete ptr;
-      ptr = nptr;
-      nptr = NULL;
-    }
-  }
-}
+AffixMgr::~AffixMgr() {}
 
 static inline void MAX(int & lhs, int rhs) 
 {
@@ -214,9 +217,11 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 {
   // io buffers
   String buf; DataPair dp;
+
+  CondsLookup conds_lookup;
  
   // open the affix file
-  affix_file = strings.dup(affpath);
+  affix_file = data_buf.dup(affpath);
   FStream afflst;
   RET_ON_ERR(afflst.open(affpath,"r"));
 
@@ -233,7 +238,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 
     if (dp.key == "SET") {
       String buf;
-      encoding = strings.dup(fix_encoding_str(dp.value, buf));
+      encoding = data_buf.dup(fix_encoding_str(dp.value, buf));
       char msg[96];
       snprintf(msg, 96, _("Expected the file to be in \"%s\" not \"%s\"."),
                lang->data_encoding(), encoding);
@@ -243,7 +248,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 
     /* parse in the flag used by the controlled compound words */
     //else if (d.key == "COMPOUNDFLAG")
-    //  compound = strings.dup(d.value);
+    //  compound = data_buf.dup(d.value);
 
     /* parse in the flag used by the controlled compound words */
     //else if (d.key == "COMPOUNDMIN")
@@ -263,7 +268,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
     int numents = 0;      // number of affentry structures to parse
     char achar='\0';      // affix char identifier
     short xpflg=0;
-    StackPtr<AffEntry> nptr;
+    AffEntry * nptr;
     {
       // split affix header line into pieces
       split(dp);
@@ -284,10 +289,13 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
       for (int j = 0; j < numents; j++) {
         getdata_pair(afflst, dp, buf);
 
-        if (affix_type == 'P')
-          nptr.reset(new PfxEntry(this));
-        else
-          nptr.reset(new SfxEntry(this));
+        if (affix_type == 'P') {
+          nptr = (AffEntry *) data_buf.alloc_bottom(sizeof(PfxEntry));
+          new (nptr) PfxEntry(this);
+        } else {
+          nptr = (AffEntry *) data_buf.alloc_bottom(sizeof(SfxEntry));
+          new (nptr) SfxEntry(this);
+        }
 
         nptr->xpflg = xpflg;
 
@@ -309,10 +317,10 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
           ParmString s0(iconv(dp.key));
           MAX(max_strip_, s0.size());
           MAX(max_strip_f[(byte)achar], s0.size());
-          nptr->strip = strings.dup(s0);
+          nptr->strip = data_buf.dup(s0);
           nptr->stripl = s0.size();
         } else {
-          nptr->strip= strings.dup("");
+          nptr->strip= data_buf.dup("");
           nptr->stripl = 0;
         }
     
@@ -320,24 +328,24 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         if (dp.key.empty()) goto error;
         // key is affix string or 0 for null
         if (dp.key != "0") {
-          nptr->appnd = strings.dup(iconv(dp.key));
+          nptr->appnd = data_buf.dup(iconv(dp.key));
           nptr->appndl = strlen(nptr->appnd);
         } else {
-          nptr->appnd  = strings.dup("");
+          nptr->appnd  = data_buf.dup("");
           nptr->appndl = 0;
         }
     
         split(dp);
         if (dp.key.empty()) goto error;
         // key is the conditions descriptions
-        encodeit(nptr,iconv(dp.key));
+        encodeit(conds_lookup, data_buf, nptr,iconv(dp.key));
     
         // now create SfxEntry or PfxEntry objects and use links to
         // build an ordered (sorted by affix string) list
         if (affix_type == 'P')
-          build_pfxlist(static_cast<PfxEntry *>(nptr.release()));
+          build_pfxlist(static_cast<PfxEntry *>(nptr));
         else
-          build_sfxlist(static_cast<SfxEntry *>(nptr.release())); 
+          build_sfxlist(static_cast<SfxEntry *>(nptr)); 
       }
     }
     continue;
@@ -420,7 +428,7 @@ PosibErr<void> AffixMgr::build_sfxlist(SfxEntry* sfxptr)
 {
   SfxEntry * ptr;
   SfxEntry * ep = sfxptr;
-  sfxptr->rappnd = (char *)strings.alloc(sfxptr->appndl + 1);
+  sfxptr->rappnd = (char *)data_buf.alloc(sfxptr->appndl + 1);
   // reverse the string
   sfxptr->rappnd[sfxptr->appndl] = 0;
   for (char * dest = sfxptr->rappnd + sfxptr->appndl - 1, * src = sfxptr->appnd;
@@ -481,7 +489,7 @@ PosibErr<void> AffixMgr::process_pfx_order()
         ptr->next_eq = ptr->next;
     }
 
-    // now clean up by adding smart search termination strings:
+    // now clean up by adding smart search termination strings
     // if you are already a superset of the previous prefix
     // but not a subset of the next, search can end here
     // so set NextNE properly
@@ -553,24 +561,67 @@ PosibErr<void> AffixMgr::process_sfx_order()
   return no_err;
 }
 
-
+// assumes the cond string is valid
+static void normalize_cond_str(char * str)
+{
+  char * s = str;
+  char * d = str;
+  while (*s) {
+    if (*s != '[') {
+      *d++ = *s++;
+    } else if (s[2] == ']') {
+      *d++ = s[1];
+      s += 3;
+    } else {
+      *d++ = *s++;
+      if (*s == '^') *d++ = *s++;
+      while (*s != ']') {
+        char * min = s;
+        for (char * i = s + 1; *i != ']'; ++i) {
+          if ((byte)*i < (byte)*min) min = i;}
+        char c = *s;
+        *d++ = *min;
+        *min = c;
+        ++s;
+      }
+      *d++ = *s++;
+    }
+  }
+  *d = '\0';
+}
 
 // takes aff file condition string and creates the
 // conds array - please see the appendix at the end of the
 // file affentry.cxx which describes what is going on here
 // in much more detail
 
-void AffixMgr::encodeit(AffEntry * ptr, const char * cs)
+static void encodeit(CondsLookup & l, ObjStack & buf, 
+                     AffEntry * ptr, char * cs)
 {
   byte c;
   int i, j, k;
+
+  // see if we already have this conds matrix
+
+  normalize_cond_str(cs);
+  CondsLookup::iterator itr = l.find(cs);
+  if (itr != l.end()) {
+    ptr->conds = *itr;
+    return;
+  }
+
+  Conds * cds = (Conds *)buf.alloc_bottom(sizeof(Conds));
+  cds->str = buf.dup(cs);
+  l.insert(cds);
+  ptr->conds = cds;
+
   int nc = strlen(cs);
   VARARRAYM(byte, mbr, nc + 1, MAXLNLEN);
 
-  // now clear the conditions array */
-  for (i=0;i<SETSIZE;i++) ptr->conds[i] = (byte) 0;
+  // now clear the conditions array
+  memset(cds->conds, 0, sizeof(cds->conds));
 
-  // now parse the string to create the conds array */
+  // now parse the string to create the conds array
   
   int neg = 0;   // complement indicator
   int grp = 0;   // group indicator
@@ -580,7 +631,7 @@ void AffixMgr::encodeit(AffEntry * ptr, const char * cs)
 
   // if no condition just return
   if (strcmp(cs,".")==0) {
-    ptr->numconds = 0;
+    cds->num = 0;
     return;
   }
 
@@ -625,14 +676,14 @@ void AffixMgr::encodeit(AffEntry * ptr, const char * cs)
           // set the proper bits in the condition array vals for those chars
           for (j=0;j<nm;j++) {
             k = (unsigned int) mbr[j];
-            ptr->conds[k] = ptr->conds[k] | (1 << n);
+            cds->conds[k] = cds->conds[k] | (1 << n);
           }
         } else {
           // complement so set all of them and then unset indicated ones
-          for (j=0;j<SETSIZE;j++) ptr->conds[j] = ptr->conds[j] | (1 << n);
+          for (j=0;j<SETSIZE;j++) cds->conds[j] = cds->conds[j] | (1 << n);
           for (j=0;j<nm;j++) {
             k = (unsigned int) mbr[j];
-            ptr->conds[k] = ptr->conds[k] & ~(1 << n);
+            cds->conds[k] = cds->conds[k] & ~(1 << n);
           }
         }
         neg = 0;
@@ -643,9 +694,9 @@ void AffixMgr::encodeit(AffEntry * ptr, const char * cs)
         // but first handle special case of . inside condition
         if (c == '.') {
           // wild card character so set them all
-          for (j=0;j<SETSIZE;j++) ptr->conds[j] = ptr->conds[j] | (1 << n);
+          for (j=0;j<SETSIZE;j++) cds->conds[j] = cds->conds[j] | (1 << n);
         } else {  
-          ptr->conds[(unsigned int) c] = ptr->conds[(unsigned int)c] | (1 << n);
+          cds->conds[(unsigned int)c] = cds->conds[(unsigned int)c] | (1 << n);
         }
       }
       n++;
@@ -655,7 +706,7 @@ void AffixMgr::encodeit(AffEntry * ptr, const char * cs)
 
     i++;
   }
-  ptr->numconds = n;
+  cds->num = n;
   return;
 }
 
@@ -847,6 +898,24 @@ WordAff * AffixMgr::expand_suffix(ParmString word, const byte * aff,
   return head;
 }
 
+CheckAffixRes AffixMgr::check_affix(ParmString word, char aff) const
+{
+  CheckAffixRes res = InvalidAffix;
+  
+  for (PfxEntry * p = pFlag[(unsigned char)aff]; p; p = p->flag_next) {
+    res = InapplicableAffix;
+    if (p->applicable(word)) return ValidAffix;
+  }
+
+  for (SfxEntry * p = sFlag[(unsigned char)aff]; p; p = p->flag_next) {
+    if (res == InvalidAffix) res = InapplicableAffix;
+    if (p->applicable(word)) return ValidAffix;
+  }
+
+  return res;
+}
+
+
 
 //////////////////////////////////////////////////////////////////////
 //
@@ -897,18 +966,33 @@ int LookupInfo::lookup (ParmString word, const SensitiveCompare * c,
 // Affix Entry
 //
 
+bool PfxEntry::applicable(SimpleString word) const
+{
+  unsigned int cond;
+  /* make sure all conditions match */
+  if ((word.size > stripl) && (word.size >= conds->num)) {
+    const byte * cp = (const byte *) word.str;
+    for (cond = 0;  cond < conds->num;  cond++) {
+      if ((conds->get(*cp++) & (1 << cond)) == 0)
+        break;
+    }
+    if (cond >= conds->num) return true;
+  }
+  return false;
+}
+
 // add prefix to this word assuming conditions hold
 SimpleString PfxEntry::add(SimpleString word, ObjStack & buf) const
 {
   unsigned int cond;
   /* make sure all conditions match */
-  if ((word.size > stripl) && (word.size >= numconds)) {
+  if ((word.size > stripl) && (word.size >= conds->num)) {
     const byte * cp = (const byte *) word.str;
-    for (cond = 0;  cond < numconds;  cond++) {
-      if ((conds[*cp++] & (1 << cond)) == 0)
+    for (cond = 0;  cond < conds->num;  cond++) {
+      if ((conds->get(*cp++) & (1 << cond)) == 0)
         break;
     }
-    if (cond >= numconds) {
+    if (cond >= conds->num) {
       /* */
       int alen = word.size - stripl;
       char * newword = (char *)buf.alloc(alen + appndl + 1);
@@ -937,7 +1021,7 @@ bool PfxEntry::check(const LookupInfo & linf, ParmString word,
 
   tmpl = word.size() - appndl;
 
-  if ((tmpl > 0) &&  (tmpl + stripl >= numconds)) {
+  if ((tmpl > 0) &&  (tmpl + stripl >= conds->num)) {
 
     // generate new root word by removing prefix and adding
     // back any characters that would have been stripped
@@ -951,14 +1035,14 @@ bool PfxEntry::check(const LookupInfo & linf, ParmString word,
     // tested
 
     cp = (byte *)tmpword;
-    for (cond = 0;  cond < numconds;  cond++) {
-      if ((conds[*cp++] & (1 << cond)) == 0) break;
+    for (cond = 0;  cond < conds->num;  cond++) {
+      if ((conds->get(*cp++) & (1 << cond)) == 0) break;
     }
 
     // if all conditions are met then check if resulting
     // root word in the dictionary
 
-    if (cond >= numconds) {
+    if (cond >= conds->num) {
       CheckInfo * lci = 0;
       CheckInfo * guess = 0;
       tmpl += stripl;
@@ -1026,16 +1110,31 @@ bool PfxEntry::check(const LookupInfo & linf, ParmString word,
   return false;
 }
 
+bool SfxEntry::applicable(SimpleString word) const
+{
+  int cond;
+  /* make sure all conditions match */
+  if ((word.size > stripl) && (word.size >= conds->num)) {
+    const byte * cp = (const byte *) (word + word.size);
+    for (cond = conds->num; --cond >=0; ) {
+      if ((conds->get(*--cp) & (1 << cond)) == 0)
+        break;
+    }
+    if (cond < 0) return true;
+  }
+  return false;
+}
+
 // add suffix to this word assuming conditions hold
 SimpleString SfxEntry::add(SimpleString word, ObjStack & buf, 
                            int limit, SimpleString orig_word) const
 {
   int cond;
   /* make sure all conditions match */
-  if ((orig_word.size > stripl) && (orig_word.size >= numconds)) {
+  if ((orig_word.size > stripl) && (orig_word.size >= conds->num)) {
     const byte * cp = (const byte *) (orig_word + orig_word.size);
-    for (cond = numconds; --cond >=0; ) {
-      if ((conds[*--cp] & (1 << cond)) == 0)
+    for (cond = conds->num; --cond >=0; ) {
+      if ((conds->get(*--cp) & (1 << cond)) == 0)
         break;
     }
     if (cond < 0) {
@@ -1076,7 +1175,7 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
 
   tmpl = word.size() - appndl;
 
-  if ((tmpl > 0)  &&  (tmpl + stripl >= numconds)) {
+  if ((tmpl > 0)  &&  (tmpl + stripl >= conds->num)) {
 
     // generate new root word by removing suffix and adding
     // back any characters that would have been stripped or
@@ -1095,8 +1194,8 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
     // this file for more info on exactly what is being
     // tested
 
-    for (cond = numconds;  --cond >= 0; ) {
-      if ((conds[*--cp] & (1 << cond)) == 0) break;
+    for (cond = conds->num;  --cond >= 0; ) {
+      if ((conds->get(*--cp) & (1 << cond)) == 0) break;
     }
 
     // if all conditions are met then check if resulting
